from __future__ import annotations

import json
import re
from collections.abc import Callable
from dataclasses import dataclass
from functools import wraps
from typing import Any, ParamSpec, cast

from .llm import llm
from .structured import T
from .tools import function_schema

P = ParamSpec("P")


def react(
    model: str,
    tools: list[Callable],
    response_format: type[T] | None = None,
    max_reacts: int = -1,
    **api_params: Any,
) -> Callable[[Callable[P, str | list[dict[str, str]] | T]], Callable[P, str | T]]:
    default_model_from_decorator = model
    default_response_format = response_format  # noqa: F841
    default_api_params_from_decorator = api_params.copy()

    def parameterized_lm_decorator(
        prompt: Callable[P, str | list[dict[str, str]] | T],
    ) -> Callable[P, str | T]:
        @wraps(prompt)
        def model_call(
            *prompt_args: P.args,
            **prompt_kwargs: P.kwargs,
        ) -> str | T:
            tools.insert(0, final_answer)

            def react(*prompt_args: P.args, acts: list[Action] = None, **prompt_kwargs: P.kwargs):  # type: ignore
                if acts is None:
                    acts = []
                output = prompt(*prompt_args, **prompt_kwargs)
                if isinstance(output, list):
                    output[-1]["content"] += "\n".join(str(a) for a in acts)
                elif isinstance(output, str):
                    return output + "\n".join(str(a) for a in acts)

            tool_names = ", ".join([f"`{tool.__name__}`" for tool in tools])
            react.__doc__ = (prompt.__doc__ if prompt.__doc__ else "") + SYSTEM_PROMPT.format(
                tool_names=tool_names, tool_descriptions=_tool_descriptions(tools)
            )

            llm_tool_call: Callable = llm(
                default_model_from_decorator,
                stop=["Observation:"],
                **default_api_params_from_decorator,
            )(react)

            def final_call(*prompt_args: P.args, acts: list[Action], **prompt_kwargs: P.kwargs):  # type: ignore
                output = prompt(*prompt_args, **prompt_kwargs)
                history = "\n".join(str(a) for a in acts)
                if isinstance(output, list):
                    output[-1]["content"] += f"\n-----{history}\n-----\n Now you must give an final answer!"
                elif isinstance(output, str):
                    return output + f"\n-----{history}\n-----\n Now you must give an final answer!"

            llm_final_call = llm(
                default_model_from_decorator,
                response_format=default_response_format,
                **default_api_params_from_decorator,
            )(final_call)  # type: ignore

            react_times = 0
            result = llm_tool_call(*prompt_args, **prompt_kwargs)
            assert isinstance(result, str)
            act = Action.from_response(result, tools)
            acts = [act]
            while not act.done and (max_reacts == -1 or react_times < max_reacts):
                result = cast(str, llm_tool_call(*prompt_args, acts=acts, **prompt_kwargs))
                act = Action.from_response(result, tools)
                react_times += 1
                acts.append(act)
            response: str | T = act.obs
            if not act.done and react_times >= max_reacts or default_response_format is not None:
                response = llm_final_call(*prompt_args, acts=acts, **prompt_kwargs)  # type: ignore
            assert not isinstance(response, list)
            return response

        model_call.__api_params__ = default_api_params_from_decorator  # type: ignore
        model_call.__func__ = prompt  # type: ignore

        return model_call  # type: ignore

    return parameterized_lm_decorator  # type: ignore[return-value]


SYSTEM_PROMPT = """You are an expert assistant who can solve any task using tools. You will be given a task to solve as best you can.
To do so, you have been given access to a list of tools: [{tool_names}].
To solve the task, you must plan forward to proceed in a series of steps, in a cycle of 'Thought:', 'Action:', 'Action Input:', and 'Observation:' sequences.

At each step, in the 'Thought:' sequence, you should first explain your reasoning towards solving the task and the tools that you want to use.
Then in the 'Action:' and 'Action Input:' sequence, you should tell system what tool to use and what arguments to use.
The action result will then appear in the 'Observation:' field, which will be available as input for the next step. And you do not need to generate this part, it will be automatically filled by the system.
In the end you have to return a final answer using the `final_answer` tool.

## Example
An fake example:
```
Task:
the input question you must answer

Thought: you should always think about what to do
Action: the action to take, should be one of [{tool_names}]
Action Input: the input to the action with JSON format representing the kwargs (e.g. {{"text": "hello world", "num_beams": 5}})
Observation: the result of the action

... (this Thought/Action/Action Input/Observation can be repeated zero or more times)

Thought: I now know the final answer
Action: final_answer
Action Input: {{"answer":"<answer>"}}
```

## Tools
{tool_descriptions}
"""


def _run_function(tools: list[Callable[..., str]], name: str, args: dict) -> str:
    for tool in tools:
        if tool.__name__ == name:
            return tool(**args)
    raise ValueError(f"Can't find tool {name}")


def final_answer(answer: str) -> str:
    """When get Final Answer, use this tool to return the answer and finishes the task."""
    return answer


@dataclass
class Action:
    thought: str
    tool: str
    args: dict
    obs: str

    @property
    def done(self) -> bool:
        if self.tool == "final_answer":
            return True
        else:
            return False

    def __str__(self) -> str:
        return self.info

    @property
    def info(self) -> str:
        if self.done:
            return f"\nThought: {self.thought}\nAction: Finish\nObservation: {self.obs}"
        return f"\nThought: {self.thought}\nAction: {self.tool}\nAction Input: {self.args}\nObservation: {self.obs}"

    @classmethod
    def from_response(cls, text: str, tools: list[Callable]) -> Action:
        special_func_token = "\nAction:"
        special_args_token = "\nAction Input:"
        special_obs_token = "\nObservation:"
        func_name, func_args = None, None
        i = text.rfind(special_func_token)
        j = text.rfind(special_args_token)
        k = text.rfind(special_obs_token)
        if 0 <= i < j:  # If the text has `Action` and `Action input`,
            if k < j:  # but does not contain `Observation`,
                # then it is likely that `Observation` is omitted by the LLM,
                # because the output text may have discarded the stop word.
                text = text.rstrip() + special_obs_token  # Add it back.
            k = text.rfind(special_obs_token)
            func_name = text[i + len(special_func_token) : j].strip().split("#")[0]
            func_args = text[j + len(special_args_token) : k].strip().split("#")[0]
            match = re.search(r"(\{.*\})", func_args, re.MULTILINE | re.IGNORECASE | re.DOTALL)
            if match:
                func_args = match.group()
            else:
                raise ValueError("Can't parse the Action Input")
            text = text[:i].strip()  # Return the response before tool call, i.e., `Thought`
            if text.startswith("Thought:"):
                text = text[len("Thought:") :]
        # return (func_name is not None), func_name, func_args, text
        if func_name is None or func_args is None:
            raise ValueError("Can't parse the response")
        tool = func_name
        args = json.loads(func_args.replace("'", '"'))
        return cls(
            thought=text,
            tool=tool,
            args=args,
            obs=_run_function(tools, tool, args),
        )


def _tool_descriptions(tools: list[Callable]) -> str:
    descriptions = []

    for tool in tools:
        schema = function_schema(tool)
        markdown = f"### {schema['name']}\n"
        markdown += f"{json.dumps(schema, ensure_ascii=False)}\n"
        descriptions.append(markdown)
    return "\n".join(descriptions)



from __future__ import annotations

from .config import config
from .llm import llm
from .react import react
from .tools import ToolResopnse, get_tools_schema

__all__ = ["llm", "config", "react", "get_tools_schema", "ToolResopnse"]
__version__ = "1.0.1"



from __future__ import annotations

import json

import pytest
from pydantic import BaseModel, ValidationError

from uglychain.structured import Mode, ResponseModel


class MockModel(BaseModel):
    foo: str


def test_response_formatter_init():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)

    assert formatter.response_type == MockModel
    assert formatter.mode == Mode.MD_JSON


def test_response_formatter_validate_response_type():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    with pytest.raises(TypeError):
        formatter.response_type = int
        formatter._validate_response_type()


def test_response_formatter_get_response_format_prompt():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    prompt = formatter.parameters
    assert "properties" in prompt.keys()
    assert "foo" in str(prompt.values())


def test_response_formatter_parse_from_response():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)

    class Choice:
        class Message:
            def __init__(self, content: str):
                self.content = content

        def __init__(self):
            self.message = self.Message("")

    choice = Choice()
    choice.message.content = '{"foo": "bar"}'
    result = formatter.parse_from_response(choice)
    assert result.foo == "bar"  # type: ignore

    invalid_response = Choice()
    invalid_response.message.content = '{"bar": "foo"}'
    with pytest.raises(ValueError):
        formatter.parse_from_response(invalid_response)


def test_response_formatter_update_markdown_json_schema_from_system_prompt():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    messages = [{"role": "system", "content": "Initial content"}]
    formatter._update_markdown_json_schema_from_system_prompt(messages)
    assert "properties" in messages[0]["content"]


def test_response_formatter_set_tools_from_params():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    api_params = {}
    formatter._set_tools_from_params(api_params)
    assert "tools" in api_params
    assert "tool_choice" in api_params


def test_response_formatter_process_parameters():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    messages = [{"role": "system", "content": "Initial content"}]
    merged_api_params = {}
    formatter.process_parameters("openai:gpt-4o", messages, merged_api_params)
    assert "tools" in merged_api_params or "response_format" in merged_api_params


def test_response_formatter_set_response_format_from_params():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    api_params = {}
    formatter._set_response_format_from_params(api_params)
    assert "response_format" in api_params
    assert api_params["response_format"]["type"] == "json_schema"


def test_response_formatter_schema():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    schema = formatter.schema
    assert "properties" in schema
    assert "foo" in schema["properties"]


def test_response_formatter_openai_schema():
    def mock_func() -> MockModel:
        return MockModel(foo="test")

    formatter = ResponseModel(mock_func)
    openai_schema = formatter.openai_schema
    assert "name" in openai_schema
    assert openai_schema["name"] == "MockModel"
    assert "parameters" in openai_schema
    assert "properties" in openai_schema["parameters"]
    assert "foo" in openai_schema["parameters"]["properties"]



from __future__ import annotations

import pytest
from pydantic import BaseModel

from uglychain.client import Client
from uglychain.llm import _get_map_keys, _get_messages, llm


class SampleModel(BaseModel):
    content: str


def test_llm_decorator(monkeypatch):
    @llm(model="test:model")
    def sample_prompt() -> str:
        return "Hello, world!"

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response"})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == "Test response"


def test_llm_decorator_with_basemodel(monkeypatch):
    @llm(model="test:model")
    def sample_prompt() -> SampleModel:
        return "Hello, world!"  # type: ignore

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": '{"content": "Test response"}'})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == SampleModel(content="Test response")


def test_llm_decorator_with_list_str(monkeypatch):
    @llm(model="test:model", n=2)
    def sample_prompt() -> str:
        return "Hello, world!"

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response 1"})},
            ),
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response 2"})},
            ),
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == ["Test response 1", "Test response 2"]


def test_llm_decorator_with_list_basemodel(monkeypatch):
    @llm(model="test:model", n=2)  # type: ignore
    def sample_prompt() -> SampleModel:
        return "Hello, world!"  # type: ignore

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": '{"content": "Test response 1"}'})},
            ),
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": '{"content": "Test response 2"}'})},
            ),
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == [SampleModel(content="Test response 1"), SampleModel(content="Test response 2")]


def test_llm_decorator_without_return_annotation(monkeypatch):
    @llm(model="test:model")
    def sample_prompt():
        return "Hello, world!"

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response"})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == "Test response"


def test_llm_decorator_with_invalid_return_type(monkeypatch):
    @llm(model="test:model")  # type: ignore
    def sample_prompt() -> int:
        return 123  # type: ignore

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response"})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    with pytest.raises(TypeError):
        sample_prompt()


def test_llm_decorator_with_empty_api_params(monkeypatch):
    @llm(model="test:model")
    def sample_prompt():
        return "Hello, world!"

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response"})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt(api_params={})  # type: ignore
    assert result == "Test response"


def test_get_messages_with_string():
    def sample_prompt():
        """System message"""
        return "User message"

    result = _get_messages("User message", sample_prompt)
    expected = [
        {"role": "system", "content": "System message"},
        {"role": "user", "content": "User message"},
    ]
    assert result == expected


def test_get_messages_without_docstring():
    def sample_prompt():  # type: ignore
        return "User message"

    result = _get_messages("User message", sample_prompt)
    expected = [
        {"role": "user", "content": "User message"},
    ]
    assert result == expected

    def sample_prompt():
        return [{"role": "user", "content": "User message"}]

    result = _get_messages([{"role": "user", "content": "User message"}], sample_prompt)
    expected = [{"role": "user", "content": "User message"}]
    assert result == expected


def test_get_messages_with_invalid_type():
    def sample_prompt():
        return 12345

    with pytest.raises(TypeError):
        _get_messages(12345, sample_prompt)  # type: ignore


def test_llm_decorator_with_inconsistent_list_lengths(monkeypatch):
    @llm(model="test:model", map_keys=["arg1", "arg2"])
    def sample_prompt(arg1: list[str], arg2: list[str]) -> str:
        return "Hello, world!"

    with pytest.raises(ValueError, match="prompt_args 和 prompt_kwargs 中的 map_key 列表必须具有相同的长度"):
        sample_prompt(["a", "b"], ["c"])


def test_llm_decorator_with_n_and_list_length_conflict(monkeypatch):
    @llm(model="test:model", map_keys=["arg1"], n=2)
    def sample_prompt(arg1: list[str]) -> str:
        return "Hello, world!"

    with pytest.raises(ValueError, match="n > 1 和列表长度 > 1 不能同时成立"):
        sample_prompt(["a", "b"])


def test_llm_decorator_with_parallel_processing(monkeypatch):
    @llm(model="test:model", map_keys=["arg1"])
    def sample_prompt(arg1: list[str]) -> str:
        "System prompt"
        return arg1  # type: ignore

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": messages[1]["content"]})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    results = sample_prompt(["a", "b", "c"])
    assert results == ["a", "b", "c"]


def test_llm_decorator_with_tools(monkeypatch):
    def mock_tool():
        pass

    @llm(model="test:model", tools=[mock_tool])
    def sample_prompt() -> str:
        return "Hello, world!"

    def mock_generate(model, messages, **kwargs):
        assert "tools" in kwargs
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": "Test response"})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == "Test response"


def test_llm_decorator_with_response_format(monkeypatch):
    @llm(model="test:model", response_format=SampleModel)
    def sample_prompt() -> SampleModel:
        return "Hello, world!"  # type: ignore

    def mock_generate(model, messages, **kwargs):
        return [
            type(
                "Choice",
                (object,),
                {"message": type("Message", (object,), {"content": '{"content": "Test response"}'})},
            )
        ]

    monkeypatch.setattr("uglychain.client.Client.generate", mock_generate)

    result = sample_prompt()
    assert result == SampleModel(content="Test response")


def test_get_map_keys_with_empty_map_keys():
    def sample_prompt(arg1: str, arg2: str):
        return "Hello, world!"

    result = _get_map_keys(sample_prompt, ("a", "b"), {}, None)
    assert result == (1, set(), set())


def test_get_map_keys_with_invalid_map_key_type():
    def sample_prompt(arg1: str):
        return "Hello, world!"

    with pytest.raises(ValueError, match="map_key 必须是列表"):
        _get_map_keys(sample_prompt, ("a",), {}, ["arg1"])



from __future__ import annotations

import inspect
from collections.abc import Callable
from concurrent.futures import ThreadPoolExecutor, as_completed
from functools import wraps
from typing import Any, ParamSpec

from .client import Client
from .config import config
from .console import Console
from .structured import ResponseModel, T
from .tools import ToolResopnse

P = ParamSpec("P")


def llm(
    model: str,
    response_format: type[T] | None = None,
    map_keys: list[str] | None = None,
    **api_params: Any,
) -> Callable[
    [Callable[P, str | list[dict[str, str]] | T]],
    Callable[P, str | list[str] | T | list[T] | ToolResopnse | list[ToolResopnse]],
]:
    """
    LLM 装饰器，用于指定语言模型和其参数。

    :param model: 模型名称
    :param api_params: API 参数，以关键字参数形式传入
    :return: 返回一个装饰器，用于装饰提示函数
    """
    default_model_from_decorator = model
    default_api_params_from_decorator = api_params.copy()

    def parameterized_lm_decorator(
        prompt: Callable[P, str | list[dict[str, str]] | T],
    ) -> Callable[P, str | list[str] | T | list[T] | ToolResopnse | list[ToolResopnse]]:
        @wraps(prompt)
        def model_call(
            *prompt_args: P.args,
            api_params: dict[str, Any] | None = None,  # type: ignore
            **prompt_kwargs: P.kwargs,
        ) -> str | list[str] | T | list[T] | ToolResopnse | list[ToolResopnse]:
            console = Console()
            # 获取被修饰函数的返回类型
            response_model = ResponseModel[T](prompt, response_format)

            # 合并装饰器级别的API参数和函数级别的API参数
            merged_api_params = config.default_api_params.copy()
            if default_api_params_from_decorator:
                merged_api_params.update(default_api_params_from_decorator)
            if api_params:
                merged_api_params.update(api_params)

            # 获取同时运行的次数
            n = merged_api_params.get("n", 1)
            # 获取模型名称
            model = merged_api_params.pop("model", default_model_from_decorator)

            console.log_model_usage_pre(model, prompt, prompt_args, prompt_kwargs)

            m, map_args_index_set, map_kwargs_keys_set = _get_map_keys(prompt, prompt_args, prompt_kwargs, map_keys)
            if m > 1 and n > 1:
                raise ValueError("n > 1 和列表长度 > 1 不能同时成立")

            def process_single_prompt(i: int) -> list[Any]:
                args = [arg[i] if j in map_args_index_set else arg for j, arg in enumerate(prompt_args)]  # type: ignore
                kwargs = {
                    key: value[i] if key in map_kwargs_keys_set else value  # type: ignore
                    for key, value in prompt_kwargs.items()
                }
                res = prompt(*args, **kwargs)  # type: ignore
                assert (
                    isinstance(res, str) or isinstance(res, list) and all(isinstance(item, dict) for item in res)
                ), ValueError("被修饰的函数返回值必须是 str 或 `messages`(list[dict[str, str]]) 类型")
                messages = _get_messages(res, prompt)
                response_model.process_parameters(model, messages, merged_api_params)

                console.log_model_usage_post_info(messages, merged_api_params)

                response = Client.generate(model, messages, **merged_api_params)

                # 从响应中解析结果
                result = [response_model.parse_from_response(choice) for choice in response]
                console.log_model_usage_post_intermediate(result)
                return result

            results = []
            console.log_progress_start(m if m > 1 else n)

            if config.use_parallel_processing:
                with ThreadPoolExecutor() as executor:
                    futures = [executor.submit(process_single_prompt, i) for i in range(m)]

                    for future in as_completed(futures):
                        results.extend(future.result())
            else:
                for i in range(m):
                    results.extend(process_single_prompt(i))

            console.log_progress_end()

            if len(results) == 0:
                raise ValueError("模型未返回任何选择")
            elif m == n == len(results) == 1:
                return results[0]
            return results

        model_call.__api_params__ = default_api_params_from_decorator  # type: ignore
        model_call.__func__ = prompt  # type: ignore

        return model_call  # type: ignore

    return parameterized_lm_decorator  # type: ignore[return-value]


def _get_map_keys(
    prompt: Callable, prompt_args: tuple, prompt_kwargs: dict, map_keys: list[str] | None
) -> tuple[int, set[int], set[str]]:
    if map_keys is None:
        return 1, set(), set()
    map_key_set: set[str] = set(map_keys)
    map_num_set: set[int] = set()
    param_mapping = inspect.getcallargs(prompt, *prompt_args, **prompt_kwargs)

    list_lengths = []
    for i, (param_name, arg_value) in enumerate(param_mapping.items()):
        if param_name in map_key_set:
            if not isinstance(arg_value, list):
                raise ValueError("map_key 必须是列表")
            if i < len(prompt_args):
                map_num_set.add(i)
                map_key_set.remove(param_name)
            list_lengths.append(len(arg_value))

    if not list_lengths:
        return 1, set(), set()

    unique_lengths = set(list_lengths)
    if len(unique_lengths) > 1:
        raise ValueError("prompt_args 和 prompt_kwargs 中的 map_key 列表必须具有相同的长度")

    return list_lengths[0], map_num_set, map_key_set


def _get_messages(prompt_ret: str | list[dict[str, str]], prompt: Callable) -> list[dict[str, str]]:
    if isinstance(prompt_ret, str):
        messages = []
        if prompt.__doc__ and prompt.__doc__.strip():
            messages.append({"role": "system", "content": prompt.__doc__})
        messages.append({"role": "user", "content": prompt_ret})
        return messages
    elif isinstance(prompt_ret, list):
        messages = prompt_ret
        if (not messages or messages[0]["role"] != "system") and prompt.__doc__ and prompt.__doc__.strip():
            messages.insert(0, {"role": "system", "content": prompt.__doc__})
        return messages
    else:
        raise TypeError("Expected prompt_ret to be a str or list of Messages")



from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Field


class Config(BaseModel):
    default_api_params: dict[str, Any] = Field(default_factory=dict, description="语言模型的默认参数。")
    use_parallel_processing: bool = Field(default=False, description="是否对语言模型使用并行处理。")
    verbose: bool = Field(default=False, description="如果为真，则启用详细日志记录。")
    show_progress: bool = Field(default=True, description="如果为真，则显示进度条。")

    def __init__(self, **data: Any) -> None:
        super().__init__(**data)


config = Config()



from __future__ import annotations

import json
from collections.abc import Callable
from typing import Any

import rich
from rich.columns import Columns
from rich.progress import Progress
from rich.table import Table, box

from .config import config


class Console:
    def __init__(self) -> None:
        self.if_log = config.verbose
        self.console = rich.console.Console()
        self.progress = Progress(disable=self.if_log or not config.show_progress)

    def log_model_usage_pre(
        self,
        model: str,
        prompt: Callable,
        args: tuple[object, ...],
        kwargs: dict[str, Any],
    ) -> None:
        """Add prompt to the logger."""
        if not self.if_log:
            return
        table = Table(title=prompt.__name__, box=box.SIMPLE)
        table.add_column("名称", justify="center", no_wrap=True)
        table.add_column("信息", justify="center")
        table.add_row("模型", model)
        table.add_row(
            "参数信息", ",".join([repr(arg) for arg in args] + [k + ":" + repr(v) for k, v in kwargs.items()])
        )
        # self.console.print(table)

    def log_progress_start(self, n: int) -> None:
        self._task_id = self.progress.add_task("模型进度", total=n)
        self.progress.start()

    def log_progress_intermediate(self) -> None:
        self.progress.update(self._task_id, advance=1)

    def log_progress_end(self) -> None:
        self.progress.stop()

    def log_model_usage_post_info(
        self,
        messages: list[dict[str, str]],
        merged_api_params: dict[str, Any],
    ) -> None:
        if not self.if_log:
            return
        table = Table(title="Prompt", box=box.SIMPLE, show_header=False)
        table.add_column("角色", justify="right", no_wrap=True)
        table.add_column("内容", justify="left")
        for message in messages:
            if message["role"] == "system":
                style = "bold green"
            elif message["role"] == "user":
                style = "italic yellow"
            else:
                style = "italic blue"
            table.add_row(message["role"], message["content"], style=style)
        self.console.print(table)
        if merged_api_params:
            if "tools" in merged_api_params:
                params_table = Table(title="Tools", box=box.SIMPLE)
                params_table.add_column("Name", justify="right", no_wrap=True)
                params_table.add_column("Parameters", justify="center")
                for tool in merged_api_params["tools"]:
                    params_table.add_row(
                        tool["function"]["name"], json.dumps(tool["function"]["parameters"], ensure_ascii=False)
                    )
                self.console.print(params_table)

    def log_model_usage_post_intermediate(self, result: list) -> None:
        self.log_progress_intermediate()
        if not self.if_log:
            return
        self.console.print(Columns([i.model_dump_json(indent=2) if not isinstance(i, str) else i for i in result]))
